@@ -32,7 +32,7 @@ double tau_post(double tau, double lambda, arma::vec theta, double rat){
 List BVAR_linear(arma::mat Yraw, 
                  arma::mat Wraw, 
                  arma::mat Exraw,
-                 int plag, 
+                 arma::uvec lags,
                  int draws, 
                  int burnin,
                  int thin,
@@ -45,11 +45,15 @@ List BVAR_linear(arma::mat Yraw,
   //----------------------------------------------------------------------------------------------------------------------
   // CONSTRUCT DATA
   //----------------------------------------------------------------------------------------------------------------------
+  int plag = lags(0);
+  int plagstar = lags(1);
+  int pmax  = max(lags);
+  
   int Traw = Yraw.n_rows;
   int M = Yraw.n_cols;
   int K = M*plag;
   int Mstar = Wraw.n_cols;
-  int Kstar = Mstar*(plag+1);
+  int Kstar = Mstar*(plagstar+1);
   
   bool texo = false; int Mex=0;
   if(Exraw.n_elem != 1){
@@ -57,14 +61,14 @@ List BVAR_linear(arma::mat Yraw,
   }
   
   mat Xraw = mlag(Yraw,plag,Traw,M);
-  mat X0 = Xraw.submat(plag,0,Traw-1,K-1);
-  mat Y = Yraw.submat(plag,0,Traw-1,M-1);
+  mat X0 = Xraw.submat(pmax,0,Traw-1,K-1);
+  mat Y = Yraw.submat(pmax,0,Traw-1,M-1);
   double T = X0.n_rows;
-  mat Wall = join_rows(Wraw,mlag(Wraw,plag,Traw,Mstar));
-  mat W0 = Wall.submat(plag,0,Traw-1,Kstar-1);
+  mat Wall = join_rows(Wraw,mlag(Wraw,plagstar,Traw,Mstar));
+  mat W0 = Wall.submat(pmax,0,Traw-1,Kstar-1);
   mat X = join_rows(X0,W0);
   if(texo){
-    mat E0 = Exraw.submat(plag,0,Traw-1,Mex-1);
+    mat E0 = Exraw.submat(pmax,0,Traw-1,Mex-1);
     X = join_rows(X,E0);
   }
   if(cons){
@@ -77,6 +81,9 @@ List BVAR_linear(arma::mat Yraw,
 
   int k = X.n_cols;
   int v = (M*(M-1))/2;
+  int n = K*M;
+  int nstar = Kstar*M;
+  
   //----------------------------------------------------------------------------------------------------------------------
   // HYPERPARAMETERS
   //----------------------------------------------------------------------------------------------------------------------
@@ -112,6 +119,7 @@ List BVAR_linear(arma::mat Yraw,
   const bool save_shrink_MN   = setting_store["shrink_MN"];
   const bool save_shrink_SSVS = setting_store["shrink_SSVS"];
   const bool save_shrink_NG   = setting_store["shrink_NG"];
+  const bool save_shrink_HS   = setting_store["shrink_HS"];
   const bool save_vola_pars   = setting_store["vola_pars"];
   //----------------------------------------------------------------------------------------------------------------------
   // OLS QUANTITIES
@@ -131,7 +139,9 @@ List BVAR_linear(arma::mat Yraw,
   }
   mat Em_draw = E_OLS; mat Em_str_draw = E_OLS;
   mat L_draw(M,M); L_draw.eye();
+  mat L_drawinv(M,M); L_drawinv.eye();
   mat Cm(K,K, fill::zeros); gen_compMat(Cm, A_OLS.rows(0,K-1), M, plag);
+  
   //----------------------------------------------------------------------------------------------------------------------
   // PRIORS
   //----------------------------------------------------------------------------------------------------------------------
@@ -154,9 +164,9 @@ List BVAR_linear(arma::mat Yraw,
   }
   for(int i=0; i < Mstar; i++){
     mat W_ = Wraw.col(i);
-    sigmas(M+i) = get_ar(W_,plag);
+    sigmas(M+i) = get_ar(W_,plagstar);
   }
-  if(prior==1) get_Vminnesota(V_prior, sigmas, shrink1, shrink2, shrink3, shrink4, cons, Mstar, plag, trend);
+  if(prior==1) get_Vminnesota(V_prior, sigmas, shrink1, shrink2, shrink3, shrink4, cons, Mstar, plag, plagstar, trend);
   
   // initialize stuff for MN prior
   mat V_prop1(k,M), V_prop2(k,M), V_prop4(k,M); 
@@ -186,15 +196,22 @@ List BVAR_linear(arma::mat Yraw,
       ii += 1;
     }
   }
+  
   // NG stuff
-  mat lambda2_A(plag+1,2,fill::zeros);
-  mat A_tau(plag+1,2); A_tau.fill(tau_theta); A_tau(0,0)=0;
-  mat A_tuning(plag+1,2); A_tuning.fill(0.43);
-  mat A_accept(plag+1,2, fill::zeros);
+  mat lambda2_A(pmax+1,2,fill::zeros);
+  mat A_tau(pmax+1,2); A_tau.fill(tau_theta); A_tau(0,0)=0;
+  mat A_tuning(pmax+1,2); A_tuning.fill(0.43);
+  mat A_accept(pmax+1,2, fill::zeros);
   // initialize stuff for NG prior
   mat A_con, V_con, P_con, A_end, V_end, P_end, A_exo, V_exo, P_exo;
   double prodlambda, dl, el, lambda, chi, psi, res;
   double unif, proposal, post_tau_prop, post_tau_curr, diff;
+  
+  // HS stuff
+  vec lambda_A_endo(n, fill::ones), nu_A_endo(n, fill::ones), 
+      lambda_A_exo(nstar, fill::ones), nu_A_exo(nstar, fill::ones),
+      lambda_L(v, fill::ones), nu_L(v, fill::ones);
+  double tau_A_endo = 1, zeta_A_endo = 1, tau_A_exo = 1, zeta_A_exo = 1, tau_L = 1, zeta_L = 1;
   //---------------------------------------------------------------
   // prior on coefficients in H matrix of VCV
   //---------------------------------------------------------------
@@ -212,10 +229,10 @@ List BVAR_linear(arma::mat Yraw,
   double L_tau = tau_theta;
   double L_tuning = 0.43;
   double L_accept = 0.0;
-  mat lambda2_Lmat(plag+1,1, fill::zeros);
-  mat L_taumat(plag+1,1, fill::zeros); 
-  mat L_accmat(plag+1,1, fill::zeros);
-  mat L_tunmat(plag+1,1, fill::zeros);
+  mat lambda2_Lmat(pmax+1,1, fill::zeros);
+  mat L_taumat(pmax+1,1, fill::zeros); 
+  mat L_accmat(pmax+1,1, fill::zeros);
+  mat L_tunmat(pmax+1,1, fill::zeros);
   //---------------------------------------------------------------
   // SV quantitites
   //---------------------------------------------------------------
@@ -291,16 +308,105 @@ List BVAR_linear(arma::mat Yraw,
   // NG
   if(save_shrink_NG){
     size_of_cube1 = {k, M, thindraws};
-    size_of_cube2 = {plag+1, 3, thindraws};
+    size_of_cube2 = {pmax+1, 3, thindraws};
   }
   arma::cube theta_store(size_of_cube1(0), size_of_cube1(1), size_of_cube1(2));
   arma::cube lambda2_store(size_of_cube2(0), size_of_cube2(1), size_of_cube2(2));
   arma::cube tau_store(size_of_cube2(0), size_of_cube2(1), size_of_cube2(2));
+  // HS
+  arma::ivec size_of_mat1 = {0, 0}, size_of_mat2 = {0, 0}, size_of_mat3 = {0, 0}, size_of_mat4 = {0, 0};
+  if(save_shrink_HS){
+    size_of_mat1 = {n, thindraws};
+    size_of_mat2 = {nstar, thindraws};
+    size_of_mat3 = {v, thindraws};
+    size_of_mat4 = {1, thindraws};
+  }
+  arma::mat lambda_A_endo_store(size_of_mat1(0), size_of_mat1(1));
+  arma::mat lambda_A_exo_store(size_of_mat2(0), size_of_mat2(1));
+  arma::mat lambda_L_store(size_of_mat3(0), size_of_mat3(1));
+  arma::mat nu_A_endo_store(size_of_mat1(0), size_of_mat1(1));
+  arma::mat nu_A_exo_store(size_of_mat2(0), size_of_mat2(1));
+  arma::mat nu_L_store(size_of_mat3(0), size_of_mat3(1));
+  arma::mat tau_A_endo_store(size_of_mat4(0), size_of_mat4(1));
+  arma::mat tau_A_exo_store(size_of_mat4(0), size_of_mat4(1));
+  arma::mat tau_L_store(size_of_mat4(0), size_of_mat4(1));
+  arma::mat zeta_A_endo_store(size_of_mat4(0), size_of_mat4(1));
+  arma::mat zeta_A_exo_store(size_of_mat4(0), size_of_mat4(1));
+  arma::mat zeta_L_store(size_of_mat4(0), size_of_mat4(1));
   //---------------------------------------------------------------------------------------------
   // MCMC LOOP
   //---------------------------------------------------------------------------------------------
   for(int irep = 0; irep < ntot; irep++){
     // Step 1: Sample coefficients
+    // Step 1a: Sample coefficients in A
+    for(int mm = 0; mm < M; mm++){ // estimate equation-by-equation
+      mat A_0    = A_draw; A_0.col(mm).fill(0);
+      mat Linv_0 = L_drawinv.rows(mm,M-1);
+      mat S_0    = exp(-0.5*Sv_draw.cols(mm,M-1));
+      mat zmat   = (Y - X * A_0) * Linv_0.t() ;
+      vec ztilde = vectorise(zmat) % vectorise(S_0);
+      mat xtilde = kron(Linv_0.col(mm), X) % repmat(vectorise(S_0),1,k);
+      mat Vinv_m = diagmat(1/V_prior.col(mm));
+      colvec a_m = A_prior.col(mm);
+      
+      mat V_p = (xtilde.t() * xtilde + Vinv_m).i();
+      mat A_p = V_p * (xtilde.t() * ztilde + Vinv_m * a_m);
+      
+      colvec rand_normal(k);
+      for(int i=0; i<k; i++){
+        rand_normal(i) = R::rnorm(0,1);
+      }
+      mat V_p_chol_lower;
+      bool chol_success = chol(V_p_chol_lower, V_p, "lower");
+      // Fall back on Rs chol if armadillo fails (it suppports pivoting)
+      if(chol_success == false){
+        NumericMatrix tmp = Rchol(V_p, true, false, -1);
+        int d = V_p.n_cols;
+        mat cholV_tmp = mat(tmp.begin(), d, d, false);
+        uvec piv = sort_index(as<vec>(tmp.attr("pivot")));
+        V_p_chol_lower = cholV_tmp.cols(piv);
+        V_p_chol_lower = V_p_chol_lower.t();
+      }
+      colvec A_m = A_p + V_p_chol_lower*rand_normal;
+      
+      A_draw.col(mm) = A_m;
+      Em_draw.col(mm) = Y.col(mm) - X * A_m;
+    }
+    // Step 1b: Sample coefficients in L
+    for(int mm = 1; mm < M; mm++){ 
+      mat eps_m = Em_draw.col(mm);
+      mat S_m   = exp(-0.5*Sv_draw.col(mm));
+      mat eps_x = Em_draw.cols(0,mm-1);
+      eps_m = eps_m % S_m;
+      eps_x = eps_x % repmat(S_m,1,mm);
+      mat Vinv_m = diagmat(1/L_prior.submat(mm,0,mm,mm-1));
+      colvec a_m = l_prior.submat(mm,0,mm,mm-1).t();
+      
+      mat V_p = (eps_x.t() * eps_x + Vinv_m).i();
+      mat A_p = V_p * (eps_x.t() * eps_m + Vinv_m * a_m);
+      
+      colvec rand_normal(mm);
+      for(int i=0; i< mm; i++){
+        rand_normal(i) = R::rnorm(0,1);
+      }
+      mat V_p_chol_lower;
+      bool chol_success = chol(V_p_chol_lower, V_p,"lower");
+      // Fall back on Rs chol if armadillo fails (it suppports pivoting)
+      if(chol_success == false){
+        NumericMatrix tmp = Rchol(V_p, true, false, -1);
+        int d = V_p.n_cols;
+        mat cholV_tmp = mat(tmp.begin(), d, d, false);
+        uvec piv = sort_index(as<vec>(tmp.attr("pivot")));
+        V_p_chol_lower = cholV_tmp.cols(piv);
+        V_p_chol_lower = V_p_chol_lower.t();
+      }
+      colvec L_m = A_p + V_p_chol_lower*rand_normal;
+      
+      L_draw.submat(mm,0,mm,mm-1) = L_m;
+    }
+    L_drawinv = L_draw.i();
+    Em_str_draw = Y * L_drawinv.t() - X * A_draw * L_drawinv.t();
+    /*
     for(int mm = 0; mm < M; mm++){ // estimate equation-by-equation
       if(mm == 0){
         mat S_m = exp(-0.5*Sv_draw.col(mm));
@@ -370,13 +476,14 @@ List BVAR_linear(arma::mat Yraw,
         Em_str_draw.col(mm) = Y.col(mm) - join_rows(X,Em_draw.cols(0,mm-1)) * A_m;
       }
     }
+     */
     //-----------------------------------------------
     // Step 2: different prior setups
     // SIMS
     if(prior == 1){
       // first shrinkage parameter (own lags)
       shrink_prop1 = exp(R::rnorm(0,scale1))*shrink1;
-      get_Vminnesota(V_prop1, sigmas, shrink_prop1, shrink2, shrink3, shrink4, cons, Mstar, plag, trend);
+      get_Vminnesota(V_prop1, sigmas, shrink_prop1, shrink2, shrink3, shrink4, cons, Mstar, plag, plagstar, trend);
       // likelihood of each coefficient
       for(int i=0; i<k; i++){
         for(int j=0; j<M; j++){
@@ -394,7 +501,7 @@ List BVAR_linear(arma::mat Yraw,
       
       // second shrinkage parameter (cross equations)
       shrink_prop2 = exp(R::rnorm(0,scale2))*shrink2;
-      get_Vminnesota(V_prop2, sigmas, shrink1, shrink_prop2, shrink3, shrink4, cons, Mstar, plag, trend);
+      get_Vminnesota(V_prop2, sigmas, shrink1, shrink_prop2, shrink3, shrink4, cons, Mstar, plag, plagstar, trend);
       // likelihood of each coefficient
       for(int i=0; i<k; i++){
         for(int j=0; j<M; j++){
@@ -412,7 +519,7 @@ List BVAR_linear(arma::mat Yraw,
       
       // fourth shrinkage parameter (weakly exogenous)
       shrink_prop4 = exp(R::rnorm(0,scale4))*shrink4;
-      get_Vminnesota(V_prop4, sigmas, shrink1, shrink2, shrink3, shrink_prop4, cons, Mstar, plag, trend);
+      get_Vminnesota(V_prop4, sigmas, shrink1, shrink2, shrink3, shrink_prop4, cons, Mstar, plag, plagstar, trend);
       // likelihood of each coefficient
       for(int i=0; i<k; i++){
         for(int j=0; j<M; j++){
@@ -466,157 +573,111 @@ List BVAR_linear(arma::mat Yraw,
     }
     // NG
     if(prior == 3){
-      // coefficients A matrix
-      for(int pp=0; pp < (plag+1); pp++){
-        if(pp==0){
-          A_con = A_draw.rows(plag*M, plag*M+Mstar-1); 
-          V_con = V_prior.rows(plag*M, plag*M+Mstar-1); 
-          P_con = A_prior.rows(plag*M, plag*M+Mstar-1);
-          
-          // sample lambda
-          dl = d_lambda + A_tau(0,1)*Mstar*M;
-          el = e_lambda + 0.5*A_tau(0,1)*accu(V_con);
-          lambda2_A(0,1) = R::rgamma(dl, 1/el);
-          
-          // sample theta
-          for(int ii=0; ii < Mstar; ii++){
-            for(int mm=0; mm < M; mm++){
-              lambda = A_tau(0,1) - 0.5;
-              psi = A_tau(0,1) * lambda2_A(0,1);
-              chi = std::pow(A_con(ii,mm)-P_con(ii,mm),2);
-              
-              res = do_rgig1(lambda, chi, psi);
-              if(res<1e-7) res = 1e-7;
-              if(res>1e+7) res = 1e+7;
-              
-              V_con(ii,mm) = res;
-            }
-          }
-          V_prior.rows(plag*M, plag*M+Mstar-1) = V_con;
-          
-          // sample tau
-          if(sample_tau_bool){
-            vec theta_vec = V_con.as_col();
-            prodlambda = as_scalar(lambda2_A.submat(0,1,pp,1));
-            
-            proposal = exp(R::rnorm(0,A_tuning(0,1)))*A_tau(0,1);
-            unif = R::runif(0,1);
-            
-            post_tau_prop = tau_post(proposal, prodlambda, theta_vec, 1);
-            post_tau_curr = tau_post(A_tau(0,1), prodlambda, theta_vec, 1);
-            diff = post_tau_prop - post_tau_curr + std::log(proposal) - std::log(A_tau(0,1));
-            if(diff > log(unif)){
-              A_tau(0,1) = proposal;
-              A_accept(0,1) += 1;
-            }
-            if(irep < 0.5*burnin){
-              if(A_accept(0,1)/irep > 0.30){A_tuning(0,1) = A_tuning(0,1)*1.01;}
-              if(A_accept(0,1)/irep < 0.15){A_tuning(0,1) = A_tuning(0,1)*0.99;}
-            }
-          }
+      // coefficients A matrix - weakly exogenous
+      for(int pp=0; pp < (plagstar+1); pp++){
+        //-------------------------------------------------------------------
+        // weakly exogenous
+        A_exo = A_draw.rows(plag*M+pp*Mstar, plag*M+(pp+1)*Mstar-1); 
+        V_exo = V_prior.rows(plag*M+pp*Mstar, plag*M+(pp+1)*Mstar-1); 
+        P_exo = A_prior.rows(plag*M+pp*Mstar, plag*M+(pp+1)*Mstar-1);
+        
+        // sample lambda
+        if(pp == 0){
+          prodlambda = 1.0;
         }else{
-          A_end = A_draw.rows((pp-1)*M, pp*M-1); 
-          V_end = V_prior.rows((pp-1)*M, pp*M-1); 
-          P_end = A_prior.rows((pp-1)*M, pp*M-1);
-          
-          // sample lambda
-          if(pp == 1){
-            prodlambda = 1.0;
-          }else{
-            prodlambda = as_scalar(prod(lambda2_A.submat(1,0,pp-1,0)));
+          prodlambda = as_scalar(prod(lambda2_A.submat(0,1,pp-1,1)));
+        }
+        dl = d_lambda + A_tau(pp,1)*M*Mstar;
+        el = e_lambda + 0.5*A_tau(pp,1)*accu(V_exo)*prodlambda;
+        lambda2_A(pp,1) = R::rgamma(dl, 1/el);
+        
+        // sample theta
+        prodlambda = as_scalar(prod(lambda2_A.submat(0,1,pp,1)));
+        for(int ii=0; ii < Mstar; ii++){
+          for(int mm=0; mm < M; mm++){
+            lambda = A_tau(pp,1) - 0.5;
+            psi = A_tau(pp,1) * prodlambda;
+            chi = std::pow(A_exo(ii,mm) - P_exo(ii,mm),2);
+            
+            res = do_rgig1(lambda, chi, psi);
+            if(res<1e-7) res = 1e-7;
+            if(res>1e+7) res = 1e+7;
+            
+            V_exo(ii,mm) = res;
           }
-          dl = d_lambda + A_tau(pp,0)*std::pow(M,2);
-          el = e_lambda + 0.5*A_tau(pp,0)*accu(V_end)*prodlambda;
-          lambda2_A(pp,0) = R::rgamma(dl, 1/el);
+        }
+        V_prior.rows(plag*M+pp*Mstar, plag*M+(pp+1)*Mstar-1) = V_exo;
+        
+        // sample tau
+        if(sample_tau_bool){
+          vec theta_vec = V_exo.as_col();
           
-          // sample theta
+          proposal = exp(R::rnorm(0,A_tuning(pp,1)))*A_tau(pp,1);
+          unif = R::runif(0,1);
+          
+          post_tau_prop = tau_post(proposal, prodlambda, theta_vec, 1);
+          post_tau_curr = tau_post(A_tau(pp,1), prodlambda, theta_vec, 1);
+          diff = post_tau_prop - post_tau_curr + std::log(proposal) - std::log(A_tau(pp,1));
+          if(diff > log(unif)){
+            A_tau(pp,1) = proposal;
+            A_accept(pp,1) += 1;
+          }
+          if(irep < 0.5*burnin){
+            if(A_accept(pp,1)/irep > 0.30){A_tuning(pp,1) = A_tuning(pp,1)*1.01;}
+            if(A_accept(pp,1)/irep < 0.15){A_tuning(pp,1) = A_tuning(pp,1)*0.99;}
+          }
+        }
+      }
+      // coefficients A matrix
+      for(int pp=0; pp < plag; pp++){
+        A_end = A_draw.rows(pp*M, (pp+1)*M-1); 
+        V_end = V_prior.rows(pp*M, (pp+1)*M-1); 
+        P_end = A_prior.rows(pp*M, (pp+1)*M-1);
+        
+        // sample lambda
+        if(pp == 0){
+          prodlambda = 1.0;
+        }else{
           prodlambda = as_scalar(prod(lambda2_A.submat(1,0,pp,0)));
-          for(int ii=0; ii < M; ii++){
-            for(int mm=0; mm < M; mm++){
-              lambda = A_tau(pp,0) - 0.5;
-              psi = A_tau(pp,0) * prodlambda;
-              chi = std::pow(A_end(ii,mm)-P_end(ii,mm),2);
-              
-              res = do_rgig1(lambda, chi, psi);
-              if(res<1e-7) res = 1e-7;
-              if(res>1e+7) res = 1e+7;
-              
-              V_end(ii,mm) = res;
-            }
-          }
-          V_prior.rows((pp-1)*M, pp*M-1) = V_end;
-          
-          // sample tau
-          if(sample_tau_bool){
-            vec theta_vec = V_end.as_col();
-            prodlambda = as_scalar(lambda2_A.submat(1,0,pp,0));
+        }
+        dl = d_lambda + A_tau(pp+1,0)*std::pow(M,2);
+        el = e_lambda + 0.5*A_tau(pp+1,0)*accu(V_end)*prodlambda;
+        lambda2_A(pp+1,0) = R::rgamma(dl, 1/el);
+        
+        // sample theta
+        prodlambda = as_scalar(prod(lambda2_A.submat(1,0,pp+1,0)));
+        for(int ii=0; ii < M; ii++){
+          for(int mm=0; mm < M; mm++){
+            lambda = A_tau(pp+1,0) - 0.5;
+            psi = A_tau(pp+1,0) * prodlambda;
+            chi = std::pow(A_end(ii,mm)-P_end(ii,mm),2);
             
-            proposal = exp(R::rnorm(0,A_tuning(pp,0)))*A_tau(pp,0);
-            unif = R::runif(0,1);
+            res = do_rgig1(lambda, chi, psi);
+            if(res<1e-7) res = 1e-7;
+            if(res>1e+7) res = 1e+7;
             
-            post_tau_prop = tau_post(proposal, prodlambda, theta_vec, 1);
-            post_tau_curr = tau_post(A_tau(pp,0), prodlambda, theta_vec, 1);
-            diff = post_tau_prop - post_tau_curr + std::log(proposal) - std::log(A_tau(pp,0));
-            if(diff > log(unif)){
-              A_tau(pp,0) = proposal;
-              A_accept(pp,0) += 1;
-            }
-            if(irep < 0.5*burnin){
-              if(A_accept(pp,0)/irep > 0.30){A_tuning(pp,0) = A_tuning(pp,0)*1.01;}
-              if(A_accept(pp,0)/irep < 0.15){A_tuning(pp,0) = A_tuning(pp,0)*0.99;}
-            }
+            V_end(ii,mm) = res;
           }
-          //-------------------------------------------------------------------
-          // weakly exogenous
-          A_exo = A_draw.rows(plag*M+pp*Mstar, plag*M+(pp+1)*Mstar-1); 
-          V_exo = V_prior.rows(plag*M+pp*Mstar, plag*M+(pp+1)*Mstar-1); 
-          P_exo = A_prior.rows(plag*M+pp*Mstar, plag*M+(pp+1)*Mstar-1);
+        }
+        V_prior.rows(pp*M, (pp+1)*M-1) = V_end;
+        
+        // sample tau
+        if(sample_tau_bool){
+          vec theta_vec = V_end.as_col();
           
-          // sample lambda
-          if(pp == 1){
-            prodlambda = 1.0;
-          }else{
-            prodlambda = as_scalar(prod(lambda2_A.submat(1,1,pp-1,1)));
+          proposal = exp(R::rnorm(0,A_tuning(pp+1,0)))*A_tau(pp+1,0);
+          unif = R::runif(0,1);
+          
+          post_tau_prop = tau_post(proposal, prodlambda, theta_vec, 1);
+          post_tau_curr = tau_post(A_tau(pp+1,0), prodlambda, theta_vec, 1);
+          diff = post_tau_prop - post_tau_curr + std::log(proposal) - std::log(A_tau(pp+1,0));
+          if(diff > log(unif)){
+            A_tau(pp+1,0) = proposal;
+            A_accept(pp+1,0) += 1;
           }
-          dl = d_lambda + A_tau(pp,1)*M*Mstar;
-          el = e_lambda + 0.5*A_tau(pp,1)*accu(V_end)*prodlambda;
-          lambda2_A(pp,1) = R::rgamma(dl, 1/el);
-          
-          // sample theta
-          prodlambda = as_scalar(prod(lambda2_A.submat(1,1,pp,1)));
-          for(int ii=0; ii < Mstar; ii++){
-            for(int mm=0; mm < M; mm++){
-              lambda = A_tau(pp,1) - 0.5;
-              psi = A_tau(pp,1) * prodlambda;
-              chi = std::pow(A_exo(ii,mm) - P_exo(ii,mm),2);
-              
-              res = do_rgig1(lambda, chi, psi);
-              if(res<1e-7) res = 1e-7;
-              if(res>1e+7) res = 1e+7;
-              
-              V_exo(ii,mm) = res;
-            }
-          }
-          V_prior.rows(plag*M+pp*Mstar, plag*M+(pp+1)*Mstar-1) = V_exo;
-          
-          // sample tau
-          if(sample_tau_bool){
-            vec theta_vec = V_exo.as_col();
-            
-            proposal = exp(R::rnorm(0,A_tuning(pp,1)))*A_tau(pp,1);
-            unif = R::runif(0,1);
-            
-            post_tau_prop = tau_post(proposal, prodlambda, theta_vec, 1);
-            post_tau_curr = tau_post(A_tau(pp,1), prodlambda, theta_vec, 1);
-            diff = post_tau_prop - post_tau_curr + std::log(proposal) - std::log(A_tau(pp,1));
-            if(diff > log(unif)){
-              A_tau(pp,1) = proposal;
-              A_accept(pp,1) += 1;
-            }
-            if(irep < 0.5*burnin){
-              if(A_accept(pp,1)/irep > 0.30){A_tuning(pp,1) = A_tuning(pp,1)*1.01;}
-              if(A_accept(pp,1)/irep < 0.15){A_tuning(pp,1) = A_tuning(pp,1)*0.99;}
-            }
+          if(irep < 0.5*burnin){
+            if(A_accept(pp+1,0)/irep > 0.30){A_tuning(pp+1,0) = A_tuning(pp+1,0)*1.01;}
+            if(A_accept(pp+1,0)/irep < 0.15){A_tuning(pp+1,0) = A_tuning(pp+1,0)*0.99;}
           }
         }
       }
@@ -665,6 +726,50 @@ List BVAR_linear(arma::mat Yraw,
         }
       }
     }
+    // HS
+    if(prior == 4){
+      //------------------------------------------
+      // coefficients H matrix
+      uvec lower_indices = trimatl_ind(size(L_draw), -1);
+      // sample local shrinkage parameter
+      for(int vv=0; vv < v; vv++){
+        lambda_L(vv) = 1/R::rgamma(1, 1/(1 / nu_L(vv) + 0.5*pow(L_draw(lower_indices(vv)),2) / tau_L));
+        nu_L(vv)     = 1/R::rgamma(1, 1/(1 + 1/lambda_L(vv)));
+      }
+      // sample global shrinkage parameter
+      tau_L  = 1/R::rgamma((v+1)/2, 1/(1/zeta_L + 0.5*sum(pow(L_draw(lower_indices),2)/lambda_L)));
+      zeta_L = 1/R::rgamma(1, 1/(1 + 1 / tau_L));
+      // update prior VCV
+      L_prior(lower_indices) = tau_L * lambda_L;
+      
+      //------------------------------------------
+      // coefficients A matrix - endogenous
+      A_end = A_draw.rows(0, plag*M-1);
+      // sample local shrinkage parameter
+      for(int nn=0; nn < n; nn++){
+        lambda_A_endo(nn) = 1.0 / R::rgamma(1, 1/(1 / nu_A_endo(nn) + 0.5*pow(A_end(nn),2) / tau_A_endo));
+        nu_A_endo(nn)     = 1.0 / R::rgamma(1, 1/(1 + 1/lambda_A_endo(nn)));
+      }
+      // sample global shrinkage parameter
+      tau_A_endo  = 1.0/R::rgamma((n+1)/2, 1/(1/zeta_A_endo + 0.5*sum(pow(vectorise(A_end),2)/lambda_A_endo)));
+      zeta_A_endo = 1.0/R::rgamma(1, 1 + 1/(1 / tau_A_endo));
+      // update prior VCV
+      V_prior.rows(0, plag*M-1) = reshape(tau_A_endo * lambda_A_endo, plag*M, M);
+      
+      //------------------------------------------
+      // coefficients A matrix - exogenous
+      A_exo = A_draw.rows(plag*M, plag*M+(plagstar+1)*Mstar-1);
+      // sample local shrinkage parameter
+      for(int nn=0; nn < nstar; nn++){
+        lambda_A_exo(nn) = 1/R::rgamma(1, 1/(1 / nu_A_exo(nn) + 0.5*pow(A_exo(nn),2) / tau_A_exo));
+        nu_A_exo(nn)     = 1/R::rgamma(1, 1/(1 + 1/lambda_A_exo(nn)));
+      }
+      // sample global shrinkage parameter
+      tau_A_exo  = 1/R::rgamma((nstar+1)/2, 1/(1/zeta_A_exo + 0.5*sum(pow(vectorise(A_exo),2)/lambda_A_exo)));
+      zeta_A_exo = 1/R::rgamma(1, 1/(1 + 1 / tau_A_exo));
+      // update prior VCV
+      V_prior.rows(plag*M, plag*M+(plagstar+1)*Mstar-1) = reshape(tau_A_exo * lambda_A_exo, (plagstar+1)*Mstar, M);
+    }
     //-----------------------------------------------
     // Step 3: Sample covariances
     for(int mm=0; mm < M; mm++){
@@ -719,6 +824,20 @@ List BVAR_linear(arma::mat Yraw,
           L_taumat(0,0) = L_tau;
           tau_store.slice((irep-burnin)/thin) = join_rows(A_tau,L_taumat);
         }
+        if(save_shrink_HS == true){
+          lambda_A_endo_store.col((irep-burnin)/thin) = lambda_A_endo;
+          lambda_A_exo_store.col((irep-burnin)/thin)  = lambda_A_exo;
+          lambda_L_store.col((irep-burnin)/thin)      = lambda_L;
+          nu_A_endo_store.col((irep-burnin)/thin)     = nu_A_endo;
+          nu_A_exo_store.col((irep-burnin)/thin)      = nu_A_exo;
+          nu_L_store.col((irep-burnin)/thin)          = nu_L;
+          tau_A_endo_store.col((irep-burnin)/thin)    = tau_A_endo;
+          tau_A_exo_store.col((irep-burnin)/thin)     = tau_A_exo;
+          tau_L_store.col((irep-burnin)/thin)         = tau_L;
+          zeta_A_endo_store.col((irep-burnin)/thin)   = zeta_A_endo;
+          zeta_A_exo_store.col((irep-burnin)/thin)    = zeta_A_exo;
+          zeta_L_store.col((irep-burnin)/thin)        = zeta_L;
+        }
       }
     }
     // check user interruption
@@ -731,12 +850,32 @@ List BVAR_linear(arma::mat Yraw,
                       Named("A_store") = A_store, 
                       Named("L_store") = L_store, 
                       Named("Sv_store") = Sv_store, 
-                      Named("shrink_store") = shrink_store,
-                      Named("gamma_store") = gamma_store, 
-                      Named("omega_store") = omega_store, 
-                      Named("theta_store") = theta_store, 
-                      Named("lambda2_store") = lambda2_store, 
-                      Named("tau_store") = tau_store, 
+                      Named("res_store") = res_store,
                       Named("pars_store") = pars_store, 
-                      Named("res_store") = res_store);
+                      Named("MN") = List::create(
+                        Named("shrink_store") = shrink_store
+                      ),
+                      Named("SSVS") = List::create(
+                        Named("gamma_store") = gamma_store, 
+                        Named("omega_store") = omega_store
+                      ),
+                      Named("NG") = List::create(
+                        Named("theta_store") = theta_store, 
+                        Named("lambda2_store") = lambda2_store, 
+                        Named("tau_store") = tau_store
+                      ),
+                      Named("HS") = List::create(
+                        Named("lambda_A_endo_store") = lambda_A_endo_store,
+                        Named("lambda_A_exo_store") = lambda_A_exo_store,
+                        Named("lambda_L_store") = lambda_L_store,
+                        Named("nu_A_endo_store") = nu_A_endo_store,
+                        Named("nu_A_exo_store") = nu_A_exo_store,
+                        Named("nu_L_store") = nu_L_store,
+                        Named("tau_A_exo_store") = tau_A_exo_store,
+                        Named("tau_A_endo_store") = tau_A_endo_store,
+                        Named("tau_L_store") = tau_L_store,
+                        Named("zeta_A_endo_store") = zeta_A_endo_store,
+                        Named("zeta_A_exo_store") = zeta_A_exo_store,
+                        Named("zeta_L_store") = zeta_L_store)
+                      );
 }
